package gus05.entity.gus.file.dirscan.info.mapext.analyzer;


import java.io.File;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Icon;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Register;
import gus05.framework.features.Retrieve;
import gus05.framework.tools.DefaultSupport;



public class MapExtAnalyzer extends DefaultSupport implements Entity, Give, Register, Retrieve, Runnable {


    public String getName()				{return "gus.file.dirscan.info.mapext.analyzer";}
    public String getCreationDate()		{return "2009.01.14";}

    
    private Service fileIcon;

    private HashMap map_icon;
    private HashMap map_ext;
    
    private PrintStream out = System.out;
    private File[] roots;
    private boolean analyzing = false;
    

    
    public MapExtAnalyzer() throws Exception
    {
    	fileIcon = Outside.service(this,"gus.icondisplay.file6");
    	
    	map_icon = new HashMap();
        map_ext = new HashMap();
    }
    
    
   
    private Map ext(File dir)
    {return (Map) map_ext.get(dir);}
    
    
    private void registerExt(File dir, Map map)
    {map_ext.put(dir,map);}
    
    
    
    
    

    public void give(Object obj) throws Exception
    {
        if(obj instanceof File)
        {handleFiles(new File[]{(File)obj});return;}
        
        if(obj instanceof File[])
        {handleFiles((File[])obj);return;}
        
        throw new Exception("Invalid data type: "+obj.getClass().getName());
    }
    

    
    
    private void handleFiles(File[] f) throws Exception
    {
        roots = null;
        for(int i=0;i<f.length;i++)
        if(!f[i].isDirectory())
            throw new Exception("Invalid directory: "+f[i].getAbsolutePath());
        roots = f;
    }
    
    
    
    public void run()
    {
        if(roots==null) return;
        if(analyzing) return;
        analyzing = true;
        for(int i=0;i<roots.length;i++)
        {
            out.println("start analyzing "+roots[i].getAbsolutePath());
            analyze(roots[i]);
        }
        analyzeCompleted();
    }
    
    
    
    
    private void analyzeCompleted()
    {
        out.println("analyze complete");
        analyzing = false;
        send(this,"analyzeCompleted()");
    }
    
    
    
    
    
    private void analyze(File dir)
    {
        HashMap extMap = new HashMap();
        
        File[] f = dir.listFiles();
        if(f==null)
        {
            registerExt(dir,extMap);
            return;
        }
        
        for(int i=0;i<f.length;i++)
        {
            if(f[i].isFile())
            {
                String ext = getFileExtension(f[i]);
                if(!ext.equals(""))
                {
                	if(!map_icon.containsKey(ext))
                        map_icon.put(ext, getIcon(f[i]));
                	increaseMap(extMap,ext);
                }	
            }
            else
            {
                analyze(f[i]);
                Map m = ext(f[i]);
                mergeMap(extMap,m);
            }
        }
        registerExt(dir,extMap);
    }
    
    
    
    

    
    
    
    
    
    
    
    

    public void register(String key, Object obj) throws Exception
    {
        if(key.equals("out"))
        {out = (PrintStream)obj;return;}
        
        throw new Exception("Unknown key: "+key);
    }



    public Object retrieve(String key) throws Exception
    {
        if(key.equals("map_icon")) return map_icon;
        if(key.equals("map_ext")) return map_ext;
        
        if(key.equals("keys"))
            return new String[]{"map_icon","map_ext"};
        
        throw new Exception("Unknown key: "+key);
    }

    
    
    
    
    
    
    
    
    
    
    
    private void increaseMap(Map map, String key)
    {
        if(map.containsKey(key))
        {
            Integer n = (Integer)map.get(key);
            map.put(key,new Integer(n.intValue()+1));
        }
        else map.put(key,new Integer(1));
    }
    
    private void increaseMap(Map map, String key, int value)
    {
        if(map.containsKey(key))
        {
            Integer n = (Integer)map.get(key);
            map.put(key,new Integer(n.intValue()+value));
        }
        else map.put(key,new Integer(value));
    }
    
    private void mergeMap(Map map, Map added)
    {
        Iterator it = added.keySet().iterator();
        while(it.hasNext())
        {
            String key = (String)it.next();
            Integer n = (Integer)added.get(key);
            increaseMap(map,key,n.intValue());
        }
    }
    
    
    
    
    private String getFileExtension(File file)
    {
        String[] n = file.getName().split("\\.");
        if(n.length>1) return n[n.length-1].toLowerCase();
        return "";
    }
    
    
    private Icon getIcon(File file)
    {
    	try	{return (Icon) fileIcon.transform(file);}
    	catch(Exception e)
    	{Outside.err(this,"getIcon(File)",e);}
    	return null;
    }
    
    

}

