package gus05.entity.gus.file.explorer.extend.mapext2;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneUI;
import javax.swing.table.TableCellRenderer;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Register;
import gus05.framework.features.Support;
import gus05.framework.features.Transform;



public class ExplorerExtendMapExt2 implements Entity, Transform, Register {


    public String getName()				{return "gus.file.explorer.extend.mapext2";}
    public String getCreationDate()		{return "2009.01.15";}


    private static Border empty = BorderFactory.createEmptyBorder();
    

    private Service synchronizer;
    private Service tableTooltip;
    private Service mapExtAnalyzer;
    
    private Map map_icon;
	private Map map_ext;
    private List types;
    
    
    
    
    private Icon extIcon(String ext)
	{
		if(map_icon.containsKey(ext))
			return (Icon) map_icon.get(ext);
		return null;
	}
    
    
    
    
    public ExplorerExtendMapExt2() throws Exception
    {
        synchronizer = Outside.service(this,"gus.graphic.scrollpane.scrollsynchronizer");
        tableTooltip = Outside.service(this,"gus.graphic.table.tooltip");
        mapExtAnalyzer = Outside.service(this,"gus.file.dirscan.info.mapext.analyzer");
        
        map_ext = (Map) mapExtAnalyzer.retrieve("map_ext");
		map_icon = (Map) mapExtAnalyzer.retrieve("map_icon");
		
        types = Arrays.asList(new String[]{"txt","properties","java","class","doc","xls","pdf","jar","zip"});
    }

    
    


    public Object transform(Object obj) throws Exception
    {
        JTree tree = (JTree)obj;
        FileTreeTableModel_extendMapExt model = new FileTreeTableModel_extendMapExt(tree,types);
        FileTreeJTable table = new FileTreeJTable(model,tree);
        
        HeaderRenderer_extendMapExt renderer = new HeaderRenderer_extendMapExt();
        table.getTableHeader().setDefaultRenderer(renderer);
        
        JScrollPane treeScroll = new JScrollPane(tree);
        JScrollPane tableScroll = new JScrollPane(table);
        synchronizer.give(new JScrollPane[]{treeScroll,tableScroll});
        
        treeScroll.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        treeScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        
        tree.setBorder(empty);
        table.setBorder(empty);
        treeScroll.setBorder(empty);
        tableScroll.setBorder(empty);
        
        tableScroll.getViewport().setBackground(tree.getBackground());
        
        JLabel treeHeader = new JLabel("Explorer");
        treeHeader.setHorizontalAlignment(SwingConstants.CENTER);
        treeHeader.setFont(treeHeader.getFont().deriveFont(Font.PLAIN));
        
        JPanel p = new JPanel(new BorderLayout());
        p.add(treeHeader, BorderLayout.NORTH);
        p.add(treeScroll, BorderLayout.CENTER);
        
        
        JSplitPane split = new JSplitPane();
        split.setDividerSize(2);
        split.setResizeWeight(1);
        split.setDividerLocation(250);
        split.setContinuousLayout(true);
        
        split.setOpaque(true);
        split.setBackground(tree.getBackground());
        
        BasicSplitPaneUI ui = (BasicSplitPaneUI) split.getUI();
        ui.getDivider().setBorder(empty);
        ui.getDivider().setBackground(tree.getBackground());
        
        
        split.setLeftComponent(p);
        split.setRightComponent(tableScroll);
        
        
        tableTooltip.give(table);        
        return split;
    }

    
    
    
    public void register(String key, Object obj) throws Exception
    {
        if(key.equals("types"))
        {types = (List)obj;return;}
        
        throw new Exception("Unknown key: "+key);
    }

    
    
    
    
    private Map findMapExt(File dir)
	{
		if(map_ext.containsKey(dir))
			return (Map) map_ext.get(dir);
		return null;
	}
    
    
    private String getFileExtension(File file)
    {
        String[] n = file.getName().split("\\.");
        if(n.length>1) return n[n.length-1].toLowerCase();
        return "";
    }
    
    
    
//    private void initializeTypes()
//    {
//        types = new ArrayList(fileIcon.keySet());
//        types.remove("dir_");
//        types.remove("driver_");
//        types.remove("system_");
//        Collections.sort(types);
//    }
    
    
    
    
    
    private class FileTreeTableModel_extendMapExt extends FileTreeTableModel implements ActionListener
    {
        private List types_;
        public FileTreeTableModel_extendMapExt(JTree tree, List types)
        {
            super(tree);
            types_ = types;
            
            if(types_ instanceof Support)
                ((Support)types_).addActionListener(this);
        }

        public String getColumnName(int y)
        {return (String)types.get(y);}
        
        public Class getColumnClass(int y)
        {return Integer.class;}
        
        public int getColumnCount()
        {return types.size();}
         
        protected Object getValueAt(File file, int columnIndex)
        {
            String type = (String)types_.get(columnIndex);
            
            if(file.isFile())
            if(getFileExtension(file).equals(type))
                return new Integer(1);
            
            Map map = findMapExt(file);
            if(map==null) return null;
            if(!map.containsKey(type)) return null;
            return map.get(type);
        }

        public void actionPerformed(ActionEvent e)
        {fireTableStructureChanged();}
    }
   
    
    
    
    
    
    
    
    
    
    
    
    private class HeaderRenderer_extendMapExt extends JLabel implements TableCellRenderer
    { 
        public HeaderRenderer_extendMapExt()
        {
            super();
            setOpaque(true);
            setHorizontalAlignment(SwingConstants.RIGHT);
        }

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
        {
            String type = (String)value;
            Icon icon = extIcon(type);
            setIcon(icon);
            
            if(icon==null)setText(type);
            else setText("");
            return this;
        }
    }
}

