package gus05.entity.gus.file.explorer.mapext.explorer2;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;
import gus05.framework.features.Provide;
import gus05.framework.features.Retrieve;
import gus05.framework.features.Support;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.Scrollable;

public class MapExtExplorer2 implements Entity, Support, Give, Provide, Graphic, Retrieve {

	
	public String getName()
	{return "gus.file.explorer.mapext.explorer2";}

	public String getCreationDate()
	{return "2009.01.15";}
	
	

	private Service explorer;
	private Service extend;
	private Service mapExtAnalyzer;
	private Service rootControl;
	private Service mapSupport;
	
	private Map map_icon;
	private Map map_ext;
	private Map extComps;
	private List extList;
	
	private JTabbedPane tabbed;
	private JPanel selectorPanel;
	private JLabel selectorLabel;
	private JTree tree;

	
	
	
	
	private Map findMapExt(File dir)
	{
		if(map_ext.containsKey(dir))
			return (Map) map_ext.get(dir);
		return null;
	}
	
	
	
	
	
	private Icon extIcon(String ext)
	{
		if(map_icon.containsKey(ext))
			return (Icon) map_icon.get(ext);
		return null;
	}

	
	
	
	private ExtComp extComp(String ext)
	{
		if(!extComps.containsKey(ext))
			extComps.put(ext,new ExtComp(ext));
		return (ExtComp) extComps.get(ext);
	}

	
	
	
	
	
	
	
	

	public MapExtExplorer2() throws Exception
	{
		explorer = Outside.service(this,"explorer");
		extend = Outside.service(this,"gus.file.explorer.extend.mapext");
		mapExtAnalyzer = Outside.service(this,"gus.file.dirscan.info.mapext.analyzer");
		rootControl = Outside.service(this,"gus.file.explorer.control.rootcontrol1");
		mapSupport = Outside.service(this,"gus.collection.support");
		
		map_ext = (Map) mapExtAnalyzer.retrieve("map_ext");
		map_icon = (Map) mapExtAnalyzer.retrieve("map_icon");
		
		extComps = new HashMap();
		extList = (List) mapSupport.transform(new ArrayList());
		extend.register("types", extList);

		tree = (JTree) explorer.retrieve("tree");
		rootControl.register("rootReceiver", explorer);
		rootControl.register("tree", tree);

		rootControl.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rootHasChanged();
			}
		});

		JComponent explorerComp = (JComponent) extend.transform(explorer.gui());
		if (explorerComp instanceof Scrollable)
			explorerComp = new JScrollPane(explorerComp);

		JPanel explorerPanel = new JPanel(new BorderLayout());
		explorerPanel.add(rootControl.gui(), BorderLayout.NORTH);
		explorerPanel.add(explorerComp, BorderLayout.CENTER);

		selectorPanel = new JPanel(new GridLayout(0, 1));
		selectorLabel = new JLabel(" ");
		selectorLabel.setFont(selectorLabel.getFont().deriveFont(Font.PLAIN));
		selectorLabel.setBorder(BorderFactory.createEtchedBorder());

		JPanel p = new JPanel(new BorderLayout());
		p.add(selectorLabel, BorderLayout.NORTH);
		p.add(selectorPanel, BorderLayout.CENTER);

		tabbed = new JTabbedPane();
		tabbed.addTab("File explorer", explorerPanel);
		tabbed.addTab("Type selector", p);
	}

	public JComponent gui()
	{return tabbed;}

	
	public void give(Object obj) throws Exception
	{rootControl.give(obj);}

	
	public Object provide() throws Exception
	{return rootControl.provide();}

	
	public Object retrieve(String key) throws Exception
	{
		if (key.equals("selectedFile"))
			return explorer.provide();
		throw new Exception("Unknown key: " + key);
	}

	
	public void addActionListener(ActionListener listener)
	{explorer.addActionListener(listener);}
	

	public void removeActionListener(ActionListener listener)
	{explorer.removeActionListener(listener);}

	
	public List listeners()
	{return explorer.listeners();}
	
	
	

	private void rootHasChanged()
	{
		try
		{
			File newRoot = (File) rootControl.provide();
			Map m = findMapExt(newRoot);
			
			if(m == null)initSelector(newRoot, null);
			else initSelector(newRoot, m.keySet());
			
			extList.clear();
		}
		catch(Exception e)
		{Outside.err(this, "rootHasChanged()", e);}
	}
	
	
	
	

	private void initSelector(File newRoot, Set set)
	{
		selectorPanel.removeAll();
		selectorLabel.setText(" root path " + newRoot.getName()
				+ " has not been analyzed yet");
		if (set == null)
			return;

		selectorLabel.setText(" " + set.size()
				+ " extensions found for root path " + newRoot.getName());

		if (set.size() <= 30)
			selectorPanel.setLayout(new GridLayout(30, 1));
		else if (set.size() <= 60)
			selectorPanel.setLayout(new GridLayout(30, 2));
		else if (set.size() <= 90)
			selectorPanel.setLayout(new GridLayout(30, 3));
		else if (set.size() <= 120)
			selectorPanel.setLayout(new GridLayout(30, 4));
		else if (set.size() <= 150)
			selectorPanel.setLayout(new GridLayout(30, 5));
		else if (set.size() <= 200)
			selectorPanel.setLayout(new GridLayout(40, 5));
		else
			selectorPanel.setLayout(new GridLayout(50, 5));

		Iterator it = set.iterator();
		while (it.hasNext()) {
			String ext = (String) it.next();
			selectorPanel.add(extComp(ext));
		}
	}

	
	
	
	
	
	
	private class ExtComp extends JCheckBox implements ItemListener
	{
		private String ext;

		public ExtComp(String ext)
		{
			this.ext = ext;
			setText(ext);
			addItemListener(this);
			setFont(getFont().deriveFont(Font.PLAIN));
			setIcon(extIcon(ext));
		}
		public void itemStateChanged(ItemEvent e)
		{
			if(isSelected())
			{
				extList.add(ext);
				setFont(getFont().deriveFont(Font.BOLD));
			}
			else
			{
				extList.remove(ext);
				setFont(getFont().deriveFont(Font.PLAIN));
			}
		}
	}
}
